//
//  DelegateLogger.java
//  EOInspector
//

import com.webobjects.foundation.*;
import com.webobjects.eoaccess.*;
import com.webobjects.eocontrol.*;
import java.io.*;
import java.sql.*;

public class DelegateLogger extends Logger implements EOObserving {
    public static NSSelector omniNotificationSelector = 
        new NSSelector("addLogForNotification",
                       new Class[] {com.webobjects.foundation.NSNotification.class});

    static DelegateLogger defaultLogger;
    NSMutableArray targets;

    public static DelegateLogger defaultLogger() {
        if (defaultLogger == null)
            defaultLogger = new DelegateLogger();
        return defaultLogger;
    }

    public static Logger loggerWithDelegateLogger() {
        return new Logger(DelegateLogger.defaultLogger());
    }

    public NSArray targets() {
        if (targets == null)
            targets = new NSMutableArray();
        return targets;
    }

    public void addLog(Object target,
                       String methodName,
                       long timestamp,
                       NSDictionary userinfo) {
        super.addLog(target, methodName, timestamp, userinfo);
        if (targets().containsObject(target) == false)
            ((NSMutableArray)targets()).addObject(target);
    }


    //
    // Omniscient observer for NotificationCenter
    //

    public void addLogForNotification(NSNotification notification) {
        if (notification.object() == null)
            addLog(this, notification.name(), notification.userInfo());
        else
            addLog(notification.object(),
                   notification.name(), notification.userInfo());
    }


    //
    // Omniscient observer for ObserverCenter
    //

    public void objectWillChange(Object object) {
        addLog(object, "objectWillChange");
    }


    //
    // EditingContext delegate methods
    //

    public void editingContextDidMergeChanges(EOEditingContext context) {
        addLog(context, "editingContextDidMergeChanges");
    }

    public NSArray editingContextShouldFetchObjects(EOEditingContext context,
                                                    EOFetchSpecification fetchSpec) {
        addLog(context, "editingContextShouldFetchObjects",
               new NSDictionary(fetchSpec, Log.FetchSpecKey));
        return null;
    }

    public boolean editingContextShouldInvalidateObject(EOEditingContext context,
                                                        EOEnterpriseObject object,
                                                        EOGlobalID globalID) {
        addLog(context, "editingContextShouldInvalidateObject",
               new NSDictionary(new Object[]{object, globalID},
                                new Object[]{Log.EnterpriseObjectKey,
                                             Log.GlobalIDKey}));
        return true;
    }

    public boolean
        editingContextShouldMergeChangesForObject(EOEditingContext context,
                                                  EOEnterpriseObject object) {
        addLog(context, "editingContextShouldMergeChangesForObject",
               new NSDictionary(object, Log.EnterpriseObjectKey));
        return true;
    }

    public boolean
        editingContextShouldPresentException(EOEditingContext context,
                                             Throwable exception) {
        addLog(context, "editingContextShouldPresentException",
               new NSDictionary(exception, Log.ExceptionKey));
        return true;
    }

    public boolean
        editingContextShouldUndoUserActionsAfterFailure(EOEditingContext context) {
        addLog(context, "editingContextShouldUndoUserActionsAfterFailure");
        return true;
    }


    public boolean editingContextShouldValidateChanges(EOEditingContext context) {
        addLog(context, "editingContextShouldValidateChanges");
        return true;
    }

    public void editingContextWillSaveChanges(EOEditingContext context) {
        addLog(context, "editingContextWillSaveChanges");
    }


    //
    // ClassDescription class delegate methods
    //

    public boolean shouldPropagateDeleteForObject(EOEnterpriseObject object,
                                                  EOEditingContext ec,
                                                  String key) {
        addLog(object, "shouldPropagateDeleteForObject",
               new NSDictionary(new Object[]{ec, key},
                                new Object[]{Log.EditingContextKey, Log.Key}));
        return true;
    }


    //
    // ModelGroup class delegate methods
    //

    public EOModelGroup defaultModelGroup() {
        addLog(EOModelGroup.class, "defaultModelGroup");
        return null;
    }


    //
    // ModelGroup delegate methods
    //

    public Class classForObjectWithGlobalID(EOEntity entity, EOGlobalID gid) {
        addLog(entity, "classForObjectWithGlobalID",
               new NSDictionary(gid, Log.GlobalIDKey));
        return null;
    }

    public Class failedToLookupClassNamed(EOEntity entity, String className) {
        addLog(entity, "failedToLookupClassNamed",
               new NSDictionary(className, Log.Key));
        return null;
    }

    public EOEntity relationshipFailedToLookupDestinationWithName(
                        EORelationship relationship, String name) {
        addLog(relationship, "relationshipFailedToLookupDestinationWithName",
               new NSDictionary(name, Log.Key));
        return null;
    }

    public EORelationship relationshipForRow(EOEntity entity,
                                             NSDictionary dictionary,
                                             EORelationship relationship) {
        addLog(entity, "relationshipForRow",
               new NSDictionary(new Object[]{dictionary, relationship},
                                new Object[]{Log.DictionaryKey, Log.RelationshipKey}));
        return relationship;
    }

    public EOEntity subEntityForEntity(EOEntity entity, NSDictionary dic) {
        addLog(entity, "subEntityForEntity",
               new NSDictionary(dic, Log.DictionaryKey));
        return null;
    }


    //
    // Adaptor delegate methods
    //

    public Object adaptorFetchedValueForValue(EOAdaptor adaptor,
                                              Object value,
                                              EOAttribute attribute) {
        addLog(adaptor, "adaptorFetchedValueForValue",
               new NSDictionary(new Object[]{value, attribute},
                                new Object[]{Log.ValueKey, Log.AttributeKey}));
        return value;
    }

    public NSDictionary reconnectionDictionaryForAdaptor(EOAdaptor adaptor) {
        addLog(adaptor, "reconnectionDictionaryForAdaptor");
        return null;
    } 


    //
    // AdaptorChannel delegate methods
    //

    public void adaptorChannelDidChangeResultSet(EOAdaptorChannel channel) {
        addLog(channel, "adaptorChannelDidChangeResultSet");
    } 

    public void adaptorChannelDidEvaluateExpression(EOAdaptorChannel channel,
                                                    EOSQLExpression expression) {
        addLog(channel, "adaptorChannelDidEvaluateExpression",
               new NSDictionary(expression, Log.SQLExpressionKey));
    }

    public void adaptorChannelDidExecuteStoredProcedure(EOAdaptorChannel channel,
                                                    EOStoredProcedure storedProcedure,
                                                    NSDictionary values) {
        addLog(channel, "adaptorChannelDidExecuteStoredProcedure",
               new NSDictionary(new Object[]{values, storedProcedure},
                                new Object[]{Log.DictionaryKey,
                                             Log.StoredProcedureKey}));
    }

    public void adaptorChannelDidFetchRow(EOAdaptorChannel channel,
                                          NSMutableDictionary row) {
        addLog(channel, "adaptorChannelDidFetchRow",
               new NSDictionary(row, Log.DictionaryKey));
    } 

    public void adaptorChannelDidFinishFetching(EOAdaptorChannel channel) {
        addLog(channel, "adaptorChannelDidFinishFetching");
    }

    public Throwable adaptorChannelDidPerformOperations(EOAdaptorChannel channel,
                                                        NSArray operation,
                                                        Throwable exception) {
        if (exception == null) {
            addLog(channel, "adaptorChannelDidPerformOperations",
                   new NSDictionary(operation, Log.AdaptorOperationKey));
        } else {
            addLog(channel, "adaptorChannelDidPerformOperations",
                   new NSDictionary(new Object[]{operation, exception},
                                    new Object[]{Log.AdaptorOperationKey,
                                                 Log.ExceptionKey}));
        }

        return null;
    }
 
    public void adaptorChannelDidSelectAttributes(EOAdaptorChannel channel,
                                                  NSArray attributes,
                                                  EOFetchSpecification fetchSpec,
                                                  boolean lock, EOEntity entity) {
        addLog(channel, "adaptorChannelDidSelectAttributes",
               new NSDictionary(new Object[]{attributes, fetchSpec,
                                             new Boolean(lock), entity},
                                new Object[]{Log.AttributeKey,
                                             Log.FetchSpecKey,
                                             Log.LockKey,
                                             Log.EntityKey}));
    } 

    public NSDictionary adaptorChannelShouldConstructStoredProcedureReturnValues(
                        EOAdaptorChannel channel) {
        addLog(channel, "adaptorChannelShouldConstructStoredProcedureReturnValues");
        return null;
    }

    public boolean adaptorChannelShouldEvaluateExpression(EOAdaptorChannel channel,
                                                          EOSQLExpression expression) {
        addLog(channel, "adaptorChannelShouldEvaluateExpression",
               new NSDictionary(expression, Log.SQLExpressionKey));
        return true;
    } 

    public NSDictionary
        adaptorChannelShouldExecuteStoredProcedure(EOAdaptorChannel channel,
                                                   EOStoredProcedure storedProcedure,
                                                   NSDictionary values) {
        addLog(channel, "adaptorChannelShouldExecuteStoredProcedure",
               new NSDictionary(new Object[]{storedProcedure, values},
                                new Object[]{Log.StoredProcedureKey,
                                             Log.DictionaryKey}));
        return null;
    } 

    public NSDictionary
        adaptorChannelShouldReturnValuesForStoredProcedure(EOAdaptorChannel channel,
                                                           NSDictionary values) {
        addLog(channel, "adaptorChannelShouldReturnValuesForStoredProcedure",
               new NSDictionary(values, Log.DictionaryKey));
        return null;
    } 

    public boolean
        adaptorChannelShouldSelectAttributes(EOAdaptorChannel channel,
                                             NSArray attributes,
                                             EOFetchSpecification fetchSpec,
                                             boolean lock, EOEntity entity) {
        addLog(channel, "adaptorChannelShouldSelectAttributes",
               new NSDictionary(new Object[]{attributes, fetchSpec,
                                             new Boolean(lock), entity},
                                new Object[]{Log.AttributeKey,
                                             Log.FetchSpecKey,
                                             Log.LockKey,
                                             Log.EntityKey}));
        return true;
    }

    public void adaptorChannelWillFetchRow(EOAdaptorChannel channel) {
        addLog(channel, "adaptorChannelWillFetchRow");
    } 

    public NSArray adaptorChannelWillPerformOperations(EOAdaptorChannel channel,
                                                       NSArray operations) {
        addLog(channel, "adaptorChannelWillPerformOperations",
               new NSDictionary(operations, Log.DatabaseOperationKey));
        return operations;
    }


    //
    // AdaptorContext delegate methods
    //

    public void adaptorContextDidBegin(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextDidBegin");
    }

    public void adaptorContextDidCommit(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextDidCommit");
    }

    public void adaptorContextDidRollback(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextDidRollback");
    }

    public boolean adaptorContextShouldBegin(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextShouldBegin");
        return true;
    }

    public boolean adaptorContextShouldCommit(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextShouldCommit");
        return true;
    }

    public boolean adaptorContextShouldConnect(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextShouldConnect");
        return true;
    }

    public boolean adaptorContextShouldRollback(EOAdaptorContext adaptorContext) {
        addLog(adaptorContext, "adaptorContextShouldRollback");
        return true;
    }


    //
    // DatabaseContext delegate methods
    //

    public void databaseContextDidFetchObjects(EODatabaseContext dbCtxt,
                                               NSArray array,
                                               EOFetchSpecification fetchSpec,
                                               EOEditingContext ec) {
        addLog(dbCtxt, "databaseContextDidFetchObjects",
               new NSDictionary(new Object[]{array, fetchSpec, ec},
                                new Object[]{Log.EnterpriseObjectKey,
                                             Log.FetchSpecKey,
                                             Log.EditingContextKey}));
    } 

    public void databaseContextDidSelectObjects(EODatabaseContext dbCtxt,
                                                EOFetchSpecification fetchSpec,
                                                EODatabaseChannel dbChannel) {
        addLog(dbCtxt, "databaseContextDidSelectObjects",
               new NSDictionary(new Object[]{fetchSpec, dbChannel},
                                new Object[]{Log.FetchSpecKey,
                                             Log.DatabaseChannelKey}));
    }

    public boolean databaseContextFailedToFetchObject(EODatabaseContext dbCtxt,
                                                      Object object,
                                                      EOGlobalID gid) {
        addLog(dbCtxt, "databaseContextFaildToFetchObject",
               new NSDictionary(new Object[]{object, gid},
                                new Object[]{Log.EnterpriseObjectKey,
                                             Log.GlobalIDKey}));
        return false;
    }

    public NSDictionary databaseContextNewPrimaryKey(EODatabaseContext dbCtxt,
                                                     Object object,
                                                     EOEntity entity) {
        addLog(dbCtxt, "databaseContextNewPrimaryKey",
               new NSDictionary(new Object[]{object, entity},
                                new Object[]{Log.EnterpriseObjectKey, Log.EntityKey}));
        return null;
    } 

    public boolean databaseContextShouldFetchArrayFault(EODatabaseContext dbCtxt,
                                                        Object object) {
        addLog(dbCtxt, "databaseContextShouldFetchArrayFault",
               new NSDictionary(object, Log.EnterpriseObjectKey));
        return true;
    }

    public boolean databaseContextShouldFetchObjectFault(EODatabaseContext dbCtxt,
                                                         Object object) {
        addLog(dbCtxt, "databaseContextShouldFetchObjectFault",
               new NSDictionary(object, Log.EnterpriseObjectKey));
        return true;
    }

    public NSArray databaseContextShouldFetchObjects(EODatabaseContext dbCtxt,
                                                     EOFetchSpecification fetchSpec,
                                                     EOEditingContext ec) {
        addLog(dbCtxt, "databaseContextShouldFetchObjects",
               new NSDictionary(new Object[]{fetchSpec, ec},
                                new Object[]{Log.FetchSpecKey, Log.EditingContextKey}));
        return null;
    } 

    public boolean databaseContextShouldHandleDatabaseException(EODatabaseContext dbCtxt,
                                                                Throwable exception) {
        addLog(dbCtxt, "databaseContextShouldHandleDatabaseException",
               new NSDictionary(exception, Log.ExceptionKey));
        return true;
    }

    public boolean
        databaseContextShouldInvalidateObjectWithGlobalID(EODatabaseContext dbCtxt,
                                                          EOGlobalID gid,
                                                          NSDictionary dic) {
        addLog(dbCtxt, "databaseContextShouldInvalidateObjectWithGlobalID",
               new NSDictionary(new Object[]{gid, dic},
                                new Object[]{Log.GlobalIDKey, Log.DictionaryKey}));
        return true;
    }

    public boolean
        databaseContextShouldLockObjectWithGlobalID(EODatabaseContext dbCtxt,
                                                    EOGlobalID gid,
                                                    NSDictionary dic) {
        addLog(dbCtxt, "databaseContextShouldLockObjectWithGlobalID",
               new NSDictionary(new Object[]{gid, dic},
                                new Object[]{Log.GlobalIDKey, Log.DictionaryKey}));
        return true;
    }

    public boolean
        databaseContextShouldRaiseExceptionForLockFailure(EODatabaseContext dbCtxt,
                                                          Throwable exception) {
        addLog(dbCtxt, "databaseContextShouldRaiseExceptionForLockFailure",
               new NSDictionary(exception, Log.ExceptionKey));
        return true;
    }

    public boolean
        databaseContextShouldSelectObjects(EODatabaseContext dbCtxt,
                                           EOFetchSpecification fetchSpec,
                                           EODatabaseChannel dbChannel) {
        addLog(dbCtxt, "databaseContextShouldSelectObjects",
               new NSDictionary(new Object[]{fetchSpec, dbChannel},
                                new Object[]{Log.FetchSpecKey,
                                             Log.DatabaseChannelKey}));
        return true;
    }

    public NSDictionary
        databaseContextShouldUpdateCurrentSnapshot(EODatabaseContext dbCtxt,
                                                   NSDictionary dic,
                                                   NSDictionary dic2,
                                                   EOGlobalID gid,
                                                   EODatabaseChannel dbChannel) {
        addLog(dbCtxt, "databaseContextShouldUpdateCurrentSnapshot",
               new NSDictionary(new Object[]{dic, dic2, gid, dbChannel},
                                new Object[]{Log.DictionaryKey, Log.Dictionary2Key,
                                             Log.GlobalIDKey, Log.DatabaseChannelKey}));
        return null;
    } 

    public boolean
        databaseContextShouldUsePessimisticLock(EODatabaseContext dbCtxt,
                                                EOFetchSpecification fetchSpec,
                                                EODatabaseChannel dbChannel) {
        boolean usesPessimisticLock;
        if (dbCtxt.updateStrategy() ==
            EODatabaseContext.UpdateWithPessimisticLocking)
            usesPessimisticLock = true;
        else
            usesPessimisticLock = false;
        addLog(dbCtxt, "databaseContextShouldUsePessimisticLock",
               new NSDictionary(new Object[]{fetchSpec, dbChannel,
                                             new Boolean(usesPessimisticLock)},
                                new Object[]{Log.FetchSpecKey,
                                             Log.DatabaseContextKey,
                                             Log.UsesPessimisticLockKey}));
        return usesPessimisticLock;
    }

    public void
        databaseContextWillFireArrayFaultForGlobalID(EODatabaseContext dbCtxt,
                                                     EOGlobalID gid,
                                                     EORelationship rel,
                                                     EOFetchSpecification fetchSpec,
                                                     EOEditingContext ec) {
        addLog(dbCtxt, "databaseContextWillFireArrayFaultForGlobalID",
               new NSDictionary(new Object[]{gid, rel, fetchSpec, ec},
                                new Object[]{Log.GlobalIDKey,
                                             Log.RelationshipKey,
                                             Log.FetchSpecKey,
                                             Log.EditingContextKey}));
    }

    public void
        databaseContextWillFireObjectFaultForGlobalID(EODatabaseContext dbCtxt,
                                                      EOGlobalID gid,
                                                      EOFetchSpecification fetchSpec,
                                                      EOEditingContext ec) {
        addLog(dbCtxt, "databaseContextWillFireObjectFaultForGlobalID",
               new NSDictionary(new Object[]{gid, fetchSpec, ec},
                                new Object[]{Log.GlobalIDKey,
                                             Log.FetchSpecKey,
                                             Log.EditingContextKey}));
    }

    /*
    public NSArray
        databaseContextWillOrderAdaptorOperations(EODatabaseContext dbCtxt,
                                                  NSArray databaseOps) {
        addLog(dbCtxt, "databaseContextWillOrderAdaptorOperations",
               new NSDictionary(databaseOps, Log.DatabaseOperationKey));
        return databaseOps;
    }
    */

    public NSArray
        databaseContextWillPerformAdaptorOperations(EODatabaseContext dbCtxt,
                                                    NSArray adaptorOps,
                                                    EOAdaptorChannel adChannel) {
        addLog(dbCtxt, "databaseContextWillPerformAdaptorOperations",
               new NSDictionary(new Object[]{adaptorOps, adChannel},
                                new Object[]{Log.AdaptorOperationKey,
                                             Log.AdaptorChannelKey}));
        return adaptorOps;
    } 

    public boolean databaseContextWillRunLoginPanelToOpenDatabaseChannel(
                    EODatabaseContext dbCtxt, EODatabaseChannel dbChannel) {
        addLog(dbCtxt, "databaseContextWillRunLoginPanelToOpenDatabaseChannel",
               new NSDictionary(dbChannel, Log.DatabaseChannelKey));
        return true;
    } 

}
